function dp = Lp(p,f,g)
%This routine implements the linear Fokker-Planck operator
%  dp = -d/dx (fp) + 1/2 d^2/dx^2 (gp)
%
%p is the probability density p(x)
%f is the deterministic drift  f(x)
%g is the random diffusion g(x)


%requirements for the stochastic differential matrix Lp are:
%  column sums must be zero 
%  off-diagonal elements are non-negative
%  diagonal elements are non-positive

%first spatial derivative term
%  
fp = f .* p;
dpleft = [diff(-fp); 0];  %case for f<0, leftward drift  dp(i) = fp(i+1)-fp(i)
dpright = [0; -diff(fp)]; %case for f>0, rightward drift  dp(i) = fp(i-1)-fp(i)
dpleft(f>=0) = 0;
dpright(f<0) = 0;
dp1 = dpleft + dpright;

%second spatial derivative term
dp2 = [0; 0.5*diff(diff(g.*p)); 0];  %dp(i) = 1/2 gp(i-1) + 1/2 gp(i+1) - gp(i)

%combine the two terms to calculate the update
dp = dp1 + dp2;

